# -*- coding: UTF-8 -*-
from setuphelpers import *
from setupdevhelpers import *
import waptlicences
import json


url = "https://www.aplim.fr/Documentation.aspx/GetDirectoryAndFiles"
targeted_text = "Moteur d'Exécution"
latest_bin = "charlemagne_moteur.exe"

def get_json_file(proxies=None):

    url="https://www.aplim.fr/Documentation.aspx/GetDirectoryAndFiles"

    headers = {
        "accept": "application/json",
        "content-type": "application/json",
        "cache-control": "no-cache",
        "pragma": "no-cache",
        "user-agent": "WAPT"
    }

    try:   
        response = requests.get(url, headers=headers, proxies=proxies)
        response.raise_for_status()
        response = response.json()

        return json.loads(response['d'])
    except:
        error('Unable to get json from aplim website !')

def find_charlemagne_downloads(json_file):
    downloads = next((item['nodes'] for item in json_file if item.get('text') == "Téléchargements"), list())
    charlemagne_item = next((item for item in downloads if item.get('text') == "Charlemagne"), dict())
    return charlemagne_item.get('nodes', list())

def find_charlemagne_software(downloads, targeted_text):
    for download in downloads:
        if targeted_text in download.get('text', ''):
            return download
    return None

def contains_letters(chaine):
    return any(c.isalpha() for c in chaine)

def get_charlemagne_version(text):
    return text.split('-')[-1].strip()

def update_package():
    # Declaring local variables
    package_updated = False
    proxies = get_proxies_from_wapt_console()
    if not proxies:
        proxies = get_proxies()

    app_name = control.name

    charlemagne_downloads = find_charlemagne_downloads(get_json_file(proxies=proxies))
    software = find_charlemagne_software(charlemagne_downloads, targeted_text)
    version = get_charlemagne_version(software['text'])
    
    download_url = software['href']

    print(f"Latest {app_name} version is: {version}")
    print(f"Download url is: {download_url}")

    # Before Aplim set up a bad naming scheme for its applications (Ex: 21.B , 9.2A ...)
    # so we check if this is not the case
    if contains_letters(version):
        print("For the moment, this application cannot be packaged due to incorrect version naming...")
        return package_updated

    for f in glob.glob('*.exe'):
        if Version(get_version_from_binary(f)) < Version(version):
            remove_file(f)

    if not isfile(latest_bin):
        print(f"Downloading: {latest_bin}")
        package_updated = True
        wget(download_url, latest_bin, proxies=proxies)
    else:
        print(f"Binary is present: {latest_bin}")

    expected_issuer = 'Aplim'
    sign_name = waptlicences.check_exe_signing_certificate(latest_bin)[0]
    if sign_name != expected_issuer:
        error(f'Bad issuer {sign_name} != {expected_issuer} ')

    # Changing version of the package
    if Version(version) > Version(control.get_software_version()):
        print(f"Software version updated (from: {control.get_software_version()} to: {version})")
        package_updated = True
    else:
        print(f"Software version up-to-date ({version})")
        
    control.set_software_version(get_version_from_binary(latest_bin))
    control.save_control_to_wapt()

    return package_updated